// -*- C++ -*-
/*!
 * @file  InMic.cpp
 * @brief MicIn Module
 * @date $Date$
 *
 * $Id$
 */

#include "InMic.h"

extern TY_RINGBUFF eRINGBUFF[RECORD_RING_MAX];
extern int nCurrentIndex;
extern WCENDIAN nEndian;

// Module specification
// <rtc-template block="module_spec">
static const char* inmic_spec[] =
  {
    "implementation_id", "InMic",
    "type_name",         "InMic",
    "description",       "MicIn Module",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.audiosize", "6400",
    "conf.default.outputWavefile", "0",
    // Widget
    "conf.__widget__.audiosize", "text",
    "conf.__widget__.outputWavefile", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
InMic::InMic(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_outWaveDataOut("outWaveData", m_outWaveData),
    m_TimeServerServicePort("TimeServerService")

    // </rtc-template>
{
	m_nSize = 8092;
	m_Voice = new char[m_nSize];
	m_element = new char[1024];
}

/*!
 * @brief destructor
 */
InMic::~InMic()
{
	delete [] m_Voice;
	delete [] m_element;
}



RTC::ReturnCode_t InMic::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("outWaveData", m_outWaveDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_TimeServerServicePort.registerConsumer("TimeServerInterface", "TimeServerInterface", m_TimeServerInterface);
  
  // Set CORBA Service Ports
  addPort(m_TimeServerServicePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("audiosize", m_audiosize, "6400");
  bindParameter("outputWavefile", m_outputWavefile, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}


RTC::ReturnCode_t InMic::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t InMic::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InMic::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t InMic::onActivated(RTC::UniqueId ec_id)
{
	eOPTCFG_MIC_BUFFER_SIZE = m_audiosize;

	m_eCounter = 0;
	m_ctWavcap.CapStart(m_audiosize, WCENDIAN_LITTLE);
	std::cout << "InMic Start\n" << std::endl;
	return RTC::RTC_OK;
}


RTC::ReturnCode_t InMic::onDeactivated(RTC::UniqueId ec_id)
{
	if(m_ctWavcap.CapEnd() == false)
		std::cout << "m_ctWavcap.CapEnd() ERROR!!" << std::endl;
	m_eCounter = 0;
	std::cout << "\nInMic Stop" << std::endl;
	return RTC::RTC_OK;
}


RTC::ReturnCode_t InMic::onExecute(RTC::UniqueId ec_id)
{
	static int read_count = 0;
	static int compair_count = 0;

	if(read_count != nCurrentIndex) {
  		read_count = nCurrentIndex;
		std::cout << "read_count = " << read_count << std::endl;

		SERVER_TM stm;

		if(false == Func_calc_timestamp( &stm, eRINGBUFF[read_count].timestamp )){
			std::cout << "!!>>> Time Get Error <<<!!" << std::endl;
			return RTC::RTC_OK;
		}
		
		if(m_nSize < eOPTCFG_MIC_BUFFER_SIZE)
 		{
 			delete [] m_Voice;
			m_nSize = eOPTCFG_MIC_BUFFER_SIZE + 16;
		 	m_Voice = new char[m_nSize];
 		}
		memcpy(&m_Voice[0], eRINGBUFF[read_count].voice, eOPTCFG_MIC_BUFFER_SIZE);
		const char* data = m_base64.Encode((char*)&m_Voice[0], eOPTCFG_MIC_BUFFER_SIZE);
		std::string senddata("<?xml version=""1.0"" encoding=""euc-jp""?>\n\n<listen>\n");
		sprintf(m_element,"<time>\n<hour>%ld</hour>\n<minutes>%ld</minutes>\n<seconds>%ld</seconds>\n<milliseconds>%ld</milliseconds>\n</time>\n", stm.tm_hour, stm.tm_min, stm.tm_sec, stm.tm_msec);
		senddata.append(m_element);
		sprintf(m_element,"<counter>%ld</counter>\n", m_eCounter++);
		senddata.append(m_element);
		senddata.append("<data>");
		senddata.append(data);
		senddata.append("</data>\n\n</listen>\n");
		std::cout << "send data size : " << eOPTCFG_MIC_BUFFER_SIZE << std::endl;

		if(m_outputWavefile == 1){
			std::ofstream ofs;
			ofs.open("mic.txt",std::ios::out);
			ofs.write(senddata.c_str(), (std::streamsize)strlen(senddata.c_str()));
			ofs.close();
		}

		m_outWaveData.data = CORBA::string_dup(senddata.data());
 		m_outWaveDataOut.write();

	}

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t InMic::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InMic::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InMic::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InMic::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InMic::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void InMicInit(RTC::Manager* manager)
  {
    coil::Properties profile(inmic_spec);
    manager->registerFactory(profile,
                             RTC::Create<InMic>,
                             RTC::Delete<InMic>);
  }
  
};

//
//
//
//
//

bool InMic::Func_calc_timestamp(SERVER_TM *rstm, long datTime)
{
	double tm1,tmb ;
	DWORD c_time;
	SERVER_TM stm;
	bool ret = m_TimeServerInterface->svr_getlocaltime(stm);

	if(false == ret) return false;

	c_time = timeGetTime();

    tm1 = stm.tm_hour * 3600000 + stm.tm_min *60000 + stm.tm_sec*1000 + stm.tm_msec;
    tmb = tm1 - (c_time-datTime);

    rstm->tm_hour  = (long)(tmb/3600000);
    rstm->tm_min   = (long)(tmb-rstm->tm_hour*3600000)/60000;
    rstm->tm_sec   = (long)(tmb-rstm->tm_hour*3600000-rstm->tm_min*60000)/1000;
    rstm->tm_msec  = (long)(tmb-(rstm->tm_hour*3600000+rstm->tm_min*60000+rstm->tm_sec*1000));

	return true;
}